$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
Get-Date | Out-File $logFile -Encoding OEM; # starting logging to file.

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
if ($config.IndexOf("<ArchiveQuotes>") -eq -1) {$ArchiveQuotes = "";}
else {$ArchiveQuotes = ($config | Select-Object -Index(($config.IndexOf("<ArchiveQuotes>"))+1)).Replace("</ArchiveQuotes>","")}; 
if ($ArchiveQuotes -eq $null) {exit(1)}; if ($ArchiveQuotes.ToLower() -ne "yes") {exit(1);} # checking if need to archive quotes, if no, exit script.

$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\Quotes\"; if (!(Test-Path -Path $quotesFolder)) {"Quotes folder not found: $quotesFolder"; exit(1)};
$fl =  @(Get-ChildItem $quotesFolder -Recurse | where {$_.extension -eq ".txt" -and (!($_.BaseName -like "*_Archive"))});

"=======================================================================================" | Out-File $logFile -Encoding OEM -Append;
"==== Starting script " + $MyInvocation.MyCommand.Name + ". File count: " + $fl.count + ". MinDate: $minDate" | Out-File $logFile -Encoding OEM -Append;
"=======================================================================================" | Out-File $logFile -Encoding OEM -Append;
$lastXdays = (Get-Date).AddDays(-50).ToString("yyyy-MM-dd"); # 50 calendar days is about 30 business days

ForEach($f in $fl) {
    $fc = @(import-csv $f.FullName -Header "Date","Price","Symbol" | Sort-Object -Property "Date"); $fileRecCount = $fc.count;
    if ($fc.count -gt 0) { # File has at least one record
        $str = ""; $strArch = ""; $newRC=0; $arcRC=0;
        For ($i = 0; $i -lt $fileRecCount; $i++) {
            $currDate = $fc[$i].Date; if ($i+1 -eq $fileRecCount) {$nextDate = $currDate} else {$nextDate = $fc[$i+1].Date;}
            if ($i -eq 0 -or $i+1 -eq $fileRecCount -or $currDate.Substring(5,2) -ne  $nextDate.Substring(5,2) -or $currDate -ge $lastXdays) { # if first or last record or if month changed, then keep record
                $str+= $fc[$i].Date + "," + $fc[$i].Price +"," + $fc[$i].Symbol + "`r`n"; $newRC++;
            }
            else { # otherwise move record to archive folder
                $strArch+=$fc[$i].Date + "," + $fc[$i].Price +"," + $fc[$i].Symbol + "`r`n"; $arcRC++;
            }
        }
        if ($str.Length -gt 1) {$str = $str.Substring(0, $str.Length-2);}
        if ($strArch.Length -gt 1) {$strArch = $strArch.Substring(0, $strArch.Length-2);}
        $fNew = $f.FullName.Replace(".txt", "_Archive.txt"); # Getting archive file name

        if ($strArch.Length -ne 0) {
            $strArch | out-file $fNew -Encoding oem -Append; # Appending to archive file
            if ($str.Length -ne 0) {$str | out-file $f.FullName -Encoding oem;} # replacing existing file, but just when there was something to archive
        }

        (Get-Date).ToString("HH:mm:ss") + " " + $f.Name.PadRight(22) + ". Was rec: " + $fileRecCount.ToString().PadLeft(5) + ". New rec: " + $newRC.ToString().PadLeft(5) + ". Archived rec: " + $arcRC.ToString().PadLeft(5) + "."  | Out-File $logFile -Encoding OEM -Append;
    }
}; # ForEach($f in $fl) {
"=======================================================================================" | Out-File $logFile -Encoding OEM -Append;
"Finished. Duration: " + (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec." | Out-File $logFile -Encoding OEM -Append;
"=======================================================================================" | Out-File $logFile -Encoding OEM -Append;
