$startTime      = GET-DATE; 
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$psDataFolder   = $scriptPathParent + "\PSData";
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Create Dates.csv file".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output

$psDataFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<PSDataFolder>"))+1)).Replace("</PSDataFolder>","");
if ($psDataFolderCfg -ne $null -and $psDataFolderCfg -ne "") {$psDataFolder = $psDataFolderCfg;} 
if (!(Test-Path $psDataFolder)) {New-Item $psDataFolder -type directory}; "PSData folder: $psDataFolder" | Out-File $logFile -Encoding OEM -Append;

# Getting Min Date from configuration file
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>",""); # Getting minimum date for quote extract
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {  # Checking if minimum date makes sense
   $str="*** Error. Minimum date if file $minDateFile should be between 1960-01-01 and 2050-01-01. Fix and restart"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1);
} 

# #######################################################################################
# ######################## Create Dates.csv file
# #######################################################################################
$outFile = $psDataFolder + "\Dates.csv"; $dToday = Get-Date; $dDate = [datetime]::ParseExact($minDate,"yyyy-MM-dd",$null); $strD = "Date";
while ($dDate -le $dToday) { $strD += "`r`n" + $dDate.ToString("yyyy-MM-dd") ; $dDate = $dDate.AddDays(1); }; 
$strD | Out-File $outFile -Encoding OEM; 
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str=(Get-Date).ToString("HH:mm:ss") + " Finished creating file Dates.csv. First date: $minDate, Last date: " + $dDate.AddDays(-1).ToString("yyyy-MM-dd") + ". Duration $duration";
$str | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$str};
# #######################################################################################

