# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Exchange Rates BoC".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\CurrExch\"; 
if (!(Test-Path $quotesFolder)) {New-Item $quotesFolder -type directory}; # Create folder
$quoteIDFileYA = $dataRootFolder + "\QuotesIntraDay\YahooCurrExchIntraday.txt"; if (Test-Path $quoteIDFileYA) { Remove-Item $quoteIDFileYA;}

$listStart = $config.IndexOf("<Currency>"); $listEnd = $config.IndexOf("</Currency>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<currency> Symbol list is empty. Exiting script.";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$currList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
$currCount=3; if ($currList.Count -le 3) {$currCount = $currList.Count;} # Maximum supported currencies - 3. But if list smaller, then dealing with less currencies
$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $currCount + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;


$cur = @("AED","ARS","AUD","BRL","CAD","CHF","CLP","CNY","DKK","EUR","GBP","HKD","IDR","ILS","INR","JPY","KRW","MXN","MYR","NOK","NZD","SEK","SGD","TWD","USD","ZAR");
$curCode = @("IEXE6506","IEXE2703","IEXE1601","IEXE2801","CAD","IEXE1101","IEXE2901","IEXE2201","IEXE0301","EUROCAE01","IEXE1201","IEXE1401","IEXE2601","IEXE5301","IEXE3001","IEXE0701","IEXE3101","IEXE2001","IEXE3201","IEXE0901","IEXE1901","IEXE1001","IEXE3701","IEXE3501","IEXE0101","IEXE3401");
$urlBase = "http://www.bankofcanada.ca/stats/results/csv?sF=LOOKUPS_@@FromFullCode@@&lP=lookup_currency_converter.php&sTF=to&sT=@@ToShortCode@@&co=1.00&dF=@@DateFrom@@&dT=@@DateTo@@"
$dateTo = Get-Date -Format "yyyy-MM-dd";

if ($currCount -le 1) {"Found just one currency, will not create currency exchange database"; exit(1);}
if ($currCount -ge 1) {if ($cur.IndexOf($currList[0]) -eq -1) {"*** Error. Currency " + $currList[0] + " is not supported."; "*** Error. Currency " + $currList[0] + " is not supported." | Out-File $logFile -Encoding OEM -Append; exit(1);}} # Check if we have first  record in currency list
if ($currCount -ge 2) {if ($cur.IndexOf($currList[1]) -eq -1) {"*** Error. Currency " + $currList[1] + " is not supported."; "*** Error. Currency " + $currList[1] + " is not supported." | Out-File $logFile -Encoding OEM -Append; exit(1);}} # Check if we have second record in currency list
if ($currCount -ge 3) {if ($cur.IndexOf($currList[2]) -eq -1) {"*** Error. Currency " + $currList[2] + " is not supported."; "*** Error. Currency " + $currList[2] + " is not supported." | Out-File $logFile -Encoding OEM -Append; exit(1);}} # Check if we have third  record in currency list

For($currFromIndex=0; $currFromIndex -lt $currCount; $currFromIndex++) {
    For($currToIndex=0; $currToIndex -lt $currCount; $currToIndex++) {
        if ($currList[$currFromIndex] -ne $currList[$currToIndex]) {
            $currFrom = $currList[$currFromIndex]; $currTo = $currList[$currToIndex]; $lastDateInFile = "";
            $currFromCode = $curCode[$cur.IndexOf($currFrom)]; $currToCode = $curCode[$cur.IndexOf($currTo)];
            $log = (Get-Date).ToString("HH:mm:ss") + " " + $currFrom + "->" + $currTo;

            $currToCodeShort = "_" + $currToCode.Replace("IEXE",""); # currencyTo code is short with 3 special overrides (next 3 lines)
            if ($currTo.ToLower() -eq "cad") { $currToCodeShort = "L_CAD";}
            if ($currTo.ToLower() -eq "eur") { $currToCodeShort = "L_EUROCAE01";}
            $symbolQuoteFile = $quotesFolder + $currFrom + "_" + $currTo + ".txt";
            $nextDate = $minDate;
            # Getting date of last quote for currency group in quote file we already have. If there is no quote file, use min date from parameter file
            if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
                $fc = @(import-csv $symbolQuoteFile -Header "Date","Rate","CurrFrom","CurrTo" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;}
                if ($lastDateInFile -ne "") {
                    $nextDate = ([datetime]::ParseExact($lastDateInFile,"yyyy-MM-dd",$null)).AddDays(1).ToString("yyyy-MM-dd"); # Adding one day to max date found in quote file - this will be next request start date
                    "  Found rate file. Looking for quotes starting from: $nextDate" | Out-File $logFile -Encoding OEM -Append;
                }
             }
            $log += " Last local quote: " + $lastDateInFile.PadRight(10) + ".";
            
            $url = $urlBase.Replace("@@FromFullCode@@",$currFromCode).Replace("@@ToShortCode@@",$currToCodeShort).Replace("@@DateFrom@@", $nextDate).Replace("@@DateTo@@",$dateTo);
            "  Url: " + $url | Out-File $logFile -Encoding OEM -Append;

            $wcD = new-object system.net.WebClient; $webpageD = ""; $reqCount++; $reqRows = 0; $lastQuote = ""; 
            try {$webpageD = $wcD.DownloadData($url);} # Get page from url
            catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; } # if attempt to get webpage failed go to next symbol

            $cfD = [System.Text.Encoding]::ASCII.GetString($webpageD); # This variable now contains downloaded currency rates
            if ($cfD.Contains("<html")) {$reqFailed++; "  No new data (returned html)" | Out-File $logFile -Encoding OEM -Append; if($verbose) {$log + " No new data received."}; continue; } # Result is html file, something went wrong, ignore result that was just returned
            
            $clD = $cfD.Split("`n"); $reqSucceed++;
             # First 3 lines just quote header; Ignore rows that says "Holiday"; Ignore empty rows; Ignore rows with quote date before $DateFrom; Ignore quotes where rate is empty value.
             ($clD | select -Skip 3| Where {$_ -notlike "*holiday*"} | Where {$_ -ne ""} | Where {$_.split(",")[1].Trim() -ne ""} |
                %{$a=$_.Split(",");if ($a[0] -ge $nextDate) {$reqRows++; $a[0]+","+$a[2] + "," + $CurrFrom + "," + $CurrTo; if ($a[0] -gt $lastQuote) {$lastQuote=$a[0];}} }) | Out-File $symbolQuoteFile -Encoding OEM -Append;

            "Done: $currFrom->$currTo (from $nextDate to $dateTo). New record count: $reqRows." | Out-File $logFile -Encoding OEM -Append;
            if ($reqRows -eq 0) {if($verbose) {$log + " No new data found."} }
            else { if($verbose) {$log + " New rec count " + $reqRows.ToString().PadLeft(4) + ". Last quote: $lastQuote" }};

        }
    }
}

$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. New records: $reqRows. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. New records: $reqRows. Duration: $duration";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;
