# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Google History".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\Quotes\";  
if (!(Test-Path $quotesFolder)) {New-Item $quotesFolder -type directory};  # Create Data folders
if (!(Test-Path ($dataRootFolder + "\QuotesIntraDay"))) {New-Item ($dataRootFolder + "\QuotesIntraDay") -type directory}; # Create Data folder

$listStart = $config.IndexOf("<Google>"); $listEnd = $config.IndexOf("</Google>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<Google> Symbol list is empty. Exiting script.";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $symbolList.count + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;

$urlBase = "http://www.google.com/finance/historical?q=@@Symbol@@&startdate=@@StartDate@@&output=csv"; # http://www.google.com/finance/historical?q=NYSEARCA%3APWC&startdate=Jan+11%2C+2014&output=csv
$mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");

ForEach($sLine in $symbolList) { # For each symbol
    $sLineArray = $sLine.Split(","); $symbol = $sLineArray[0]; $symbolMinDate = $sLineArray[1]; $symbolMaxDate = $sLineArray[2]; $lastDateInFile = "";
    $symbolQuoteFile = $quotesFolder + "_" + $symbol.Replace(":","_").Replace("^","_").Replace("&","_") + "_" + ".txt"; # Symbol file were quotes will be saved. Adding "_" in front of symbol names as sometimes sequences like \PRN or \CON means something....
    if ($symbolMinDate -ne "" -and $symbolMinDate -ne $null -and ($symbolMinDate.length -ne 10 -or $symbolMinDate -le "1950-01-01" -or $symbolMinDate -gt "2050-01-01")) {$str=" ***Error. Symbol $symbol MinDate specified is not correct. Value provided '$symbolMinDate'. Expected date in format YYYY-MM-DD"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1); }
    if ($symbolMaxDate -ne "" -and $symbolMaxDate -ne $null -and ($symbolMaxDate.length -ne 10 -or $symbolMaxDate -le "1950-01-01" -or $symbolMaxDate -gt "2050-01-01")) {$str=" ***Error. Symbol $symbol MaxDate specified is not correct. Value provided '$symbolMinDate'. Expected date in format YYYY-MM-DD"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1); }
    if ($symbolMinDate -eq "" -or  $symbolMinDate -eq $null -or  $symbolMinDate -lt $minDate) {$nextDate = $minDate;} else {$nextDate = $symbolMinDate;}
    if ($symbolMaxDate -eq "" -or  $symbolMaxDate -eq $null) {$symbolMaxDate = (Get-Date).AddDays(1).ToString("yyyy-MM-dd");}
    "Symbol: $symbol" | Out-File $logFile -Encoding OEM -Append; $log = (Get-Date).ToString("HH:mm:ss") + " " + $symbol.PadRight(20);

    # Getting date of last quote for symbol in quote file we already have. If there is no quote file, use min date from parameter file
    if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
        $fc = @(import-csv $symbolQuoteFile -Header "Date","Close","Symbol" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;} # Finding max quote date in file.Format will be YYYY-MM-DD
        if ($lastDateInFile -eq $null) {$lastDateInFile = "";}
        if ($lastDateInFile -ne "") { # File exists and max date in the file found
            $nextDate = ([datetime]::ParseExact($lastDateInFile,"yyyy-MM-dd",$null)).AddDays(1).ToString("yyyy-MM-dd"); # Adding one day to max date found in quote file - this will be next request start date
            "  Found quote file. Looking for quotes starting from: $nextDate" | Out-File $logFile -Encoding OEM -Append;
        }
        else {$nextDate = ([datetime]::ParseExact($symbolMaxDate,"yyyy-MM-dd",$null)).AddDays(10).ToString("yyyy-MM-dd");} # Empty file means that extract already got AVAILABLE data up to max date, so no need to load more
    }
    $log += " Last local quote: " + $lastDateInFile.PadRight(10) + ".";
    if ($nextDate -gt $symbolMaxDate) { # For this symbol we already have localy data up to Max date.
        "  We already have data up to maximum configuration date of $symbolMaxDate. Will not request new data for this symbol." | Out-File $logFile -Encoding OEM -Append; 
        #$log + " Local file has all necessary data."; 
        continue; 
    } 
    
    $symbolMaxDateAdj = ([datetime]::ParseExact($symbolMaxDate,"yyyy-MM-dd",$null)).AddDays(10).ToString("yyyy-MM-dd"); # Will try to save up to 10 more days than request, because specific day quote might not be available (ex: weekend)
    $year = $nextDate.Substring(0,4); $month = [int]$nextDate.Substring(5,2) - 1; $day = $nextDate.Substring(8,2); $mthNotShifted = [int] $month+1; # Spliting date into parts. Month is 0 based

     # ========== Get quotes from website - Start ==============
     $startDate=$mths[$month]+"+"+[int] $day+"%2C+"+$year; # Jan+1%2C+2015
     $url = $urlBase.Replace("@@Symbol@@",$symbol).Replace("@@StartDate@@",$startDate);
    "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append; $webpage = ""; $reqRows=0; $lastQuote = ""; 
     $wc = new-object system.net.WebClient; $reqCount++;
     try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
     catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; } # if attempt to get webpage failed go to next symbol
     
     $quotesTxt = [System.Text.Encoding]::ASCII.GetString($webpage); $quotesTxt = $quotesTxt.Replace("???",""); # This variable now contains downloaded quotes
     if ($quotesTxt.length -le 45) { $reqFailed++; "  $symbol - Not Found (return empty file) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; } # Check if data received makes sense. If request size is less than 45 bytes (header length is 42), then there is something wrong with data received. Ignoring it, going to next symbol
     if ($quotesTxt.Contains("<html>")) {$reqFailed++; "  $symbol - Not Found (return html) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue;} # Result is html file, ignore such file
     $reqSucceed++; # If reached this point, that means symbol request was successful, go next to copy received data into local file
     # ========== Get quotes from website - End. Quote is in $quotesTxt ==============

     $ql = $quotesTxt.Split("`n"); #Splitting text file into lines
     ForEach($q in $ql) { # for each quote line
        if ($q.trim() -ne "" -and !$q.StartsWith("Date")) { #ignore quote lines without data
            $qParts = $q.Split(","); $qDate=$qParts[0]; $qPrice = $qParts[4].REPLACE("-","0");
            if ($qDate.Length -eq 8) {$qDate="0"+$qDate}; # quote date adding leading zero if required 5/06/2015 -> 05/06/2015
            $mthName = $qDate.Substring(3,3); $qMonth= "0"+($mths.IndexOf($mthName) +1); $qMonth = $qMonth.Substring($qMonth.length-2,2); # converting month name into number: Feb->02
            if($qDate.Substring(7,2) -le "50") {$qYear="20"+$qDate.Substring(7,2)} else {$qYear="19"+$qDate.Substring(7,2)} # converting 2char year into 4 char year YY->YYYY (adding century)
            $qDateFormatted = $qYear + "-" + $qMonth +"-" + $qDate.Substring(0,2);
            if($qDateFormatted -le $symbolMaxDateAdj) { # If quote date is before or equal to maximum symbol date
                $qDateFormatted + "," + $qPrice + "," + $symbol | Out-File $symbolQuoteFile -Encoding OEM -Append;
                if ($qDateFormatted -gt $lastQuote) {$lastQuote = $qDateFormatted ;} $reqRows++;
            }
        }
     }

     "  Added new quotes to file: " + $symbolQuoteFile | Out-File $logFile -Encoding OEM -Append; 
     "Done: $symbol (from: $nextDate). Record count: $reqRows. Last quote: $lastQuote" | Out-File $logFile -Encoding OEM -Append; ;
     if ($verbose) {$log + " New rec count " + $reqRows.ToString().PadLeft(4) + ". Last quote: $lastQuote"};
}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed.. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Duration: $duration";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;

