# 2016-Nov-20. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Google Intraday".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\Quotes\"; if (!(Test-Path -Path $quotesFolder)) {"Quotes folder not found: $quotesFolder"; exit(1)};
$quoteIDFileGO = $dataRootFolder + "\QuotesIntraDay\GoogleIntraday.txt"; if (Test-Path $quoteIDFileGO) { Remove-Item $quoteIDFileGO;} # If file exists, need to delete it

$sl1IsEmpty = $false; $sl2IsEmpty = $false; $symbolList = @();
$listStart = $config.IndexOf("<Google>"); $listEnd = $config.IndexOf("</Google>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<Google> Symbol list is empty. ";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; $sl1IsEmpty=$true;}
if (!$sl1IsEmpty) {$symbolList += @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));} #list of symbols we will work on

$listStart = $config.IndexOf("<GoogleWeb>"); $listEnd = $config.IndexOf("</GoogleWeb>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<GoogleWeb> Symbol list is empty. ";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; $sl2IsEmpty=$true;}
if (!$sl2IsEmpty) {$symbolList += @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));} #list of symbols we will work on

if ($sl1IsEmpty -and $sl2IsEmpty) {$str="Go & GoW empty. Exiting script. ";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($symbolList | ?{$a=$_.split(","); ($a[3] -eq $null -or $a[3] -eq "" -or $a[3] -eq "Y") -and ($a[2] -eq "" -or $a[2] -eq $null)}); # by default if value is empty or y, then get intra-day quotes. Also MaxDate for symbol should not be specified

$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $symbolList.count + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;

$urlBase = "https://www.google.com/finance?q=@@Symbol@@";   $mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");


ForEach($sLine in $symbolList) { # For each symbol
    $sLineArray = $sLine.Split(","); $symbol = $sLineArray[0]; $lastDateInFile = ""; #$symbolMinDate = $sLineArray[1]; $symbolMaxDate = $sLineArray[2]; 
    $symbolQuoteFile = $quotesFolder + "_" + $symbol.Replace(":","_").Replace("^","_").Replace("&","_") + "_" + ".txt"; # Symbol file were quotes will be saved. Adding "_" in front of symbol names as sometimes sequences like \PRN or \CON means something....
    "Symbol: $symbol" | Out-File $logFile -Encoding OEM -Append; $log = (Get-Date).ToString("HH:mm:ss") + " " + $symbol.PadRight(20);

    # Getting date of last quote for symbol in quote file we already have. If there is no quote file, use min date from parameter file
    if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
        $fc = @(import-csv $symbolQuoteFile -Header "Date","Close","Symbol" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;} # Finding max quote date in file.Format will be YYYY-MM-DD
        if ($lastDateInFile -eq $null) {$lastDateInFile = "";}
    }
    "  Last local quote: " + $lastDateInFile.PadRight(10) + "." | Out-File $logFile -Encoding OEM -Append;

     # ========== Get latest quote from website - Start ==============
     $url = $urlBase.Replace("@@Symbol@@",$symbol); 
    "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append; $webpage = ""; $reqRows=0; $lastQuote = ""; 
     $wc = new-object system.net.WebClient; $reqCount++;
     try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
     catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; } # if attempt to get webpage failed go to next symbol
     
     $quotesTxt = [System.Text.Encoding]::ASCII.GetString($webpage); $quotesTxt = $quotesTxt.Replace("???",""); # This variable now contains downloaded quotes

    # ---------------------------------------------
    # Identifying price
    # ---------------------------------------------
    #Price is in this string: <span class="pr"><span id="ref_696698_l">15.24</span>
    $startStr = "<span class=`"pr`">"; $endStr = "</span>";
    
    if ($quotesTxt.Contains($startStr)) {$quotesTxt = $quotesTxt.Substring($quotesTxt.IndexOf($startStr) + $startStr.Length,500)} # This now includes price and date
    else {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '<span class='pr'>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received.";} continue; } # if that symbol has no quotes, then record errors

    if (!$quotesTxt.Contains($endStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '</span>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; }# if that symbol has no quotes, then record errors
    $priceTxt = $quotesTxt.Substring(0,$quotesTxt.IndexOf($endStr));

    $startStr = "<span id=";
    if (!$priceTxt.Contains($startStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '<span id=' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; }# if that symbol has no quotes, then record errors
    $priceTxt = $priceTxt.Substring($priceTxt.IndexOf($startStr) + $startStr.Length);

     $startStr = ">";
     if (!$priceTxt.Contains($startStr)) {$reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no '>' in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue;} # if that symbol has no quotes, then record errors
     $priceTxt = $priceTxt.Substring($priceTxt.IndexOf($startStr) + $startStr.Length).Replace(",","");
     "  Found price: $priceTxt" | Out-File $logFile -Encoding OEM -Append;
     $reqSucceed++;

    # ---------------------------------------------
    # Identifying date
    # ---------------------------------------------
    # Date: <span class=nwp>Nov 21 - Close</span>  # or date: <div id="ref_22144_elt">Nov 21, 7:59PM EST&nbsp;&nbsp;<div class=mdata-dis>  # or date: <span class=nwp>Real-time:&nbsp;<span id="ref_1982937_ltt">1:53PM EST
    $dateTxt = $quotesTxt.Replace("`r","").Replace("`n","");  $dateFound = $true;
    $startStr = "<span class=nwp>"; $dateVal = (Get-Date).ToString("yyyy-MM-dd");
    if ($dateTxt.Contains($startStr)) { $dateVal = $dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length,7).Replace("`r","").Replace("`n",""); # This now includes date
          "  Date found in <span class=nwp>: '$dateVal'" | Out-File $logFile -Encoding OEM -Append;
    }
    else {
         $startStr = "_elt`">";
         if ($dateTxt.Contains($startStr)) {
              $dateVal = $dateTxt.Substring($dateTxt.IndexOf($startStr) + $startStr.Length,7).Replace("`r","").Replace("`n",""); # This now includes date
          "  Date found after string '_elt': '$dateVal'" | Out-File $logFile -Encoding OEM -Append;
         }
         else {$dateFound = $false;  "  Date found not found, using todays date." | Out-File $logFile -Encoding OEM -Append;}
    }
    
    $date = (Get-Date).ToString("yyyy-MM-dd");
    $mthName = $dateVal.Substring(0,3); # Date is like: Nov 20,... - so first 3 characters contain month name
    if ($dateFound -and $mths.IndexOf($mthName) -ne -1) {
        $year = (Get-Date).ToString("yyyy");
        $month = ($mths.IndexOf($mthName) + 1).ToString(); if ($mths.IndexOf($mthName) + 1 -lt 10) {$month = "0" + $month;}
        $day = $dateVal.Substring(4,2).Replace(" ","").Replace(",","");
        if ($day -match "^[\d]+$") {if ($day.Length -eq 1) {$day = "0" + $day;} $date = $year + "-" + $month + "-" + $day;} 
    }

    $dayOfWeek = ([datetime]::ParseExact($date,"yyyy-MM-dd",$null)).DayofWeek.ToString().Substring(0,3)

    if($date -gt $lastDateInFile -and $dayOfWeek -ne "Sun" -and $dayOfWeek -ne "Sat") { # If quote date is after or equal to date we already have
        $dateAsDate = [datetime]::ParseExact($date,"yyyy-MM-dd",$null)

        "  $date,$priceTxt,$symbol" | Out-File $logFile -Encoding OEM -Append;
        $date + "," + $priceTxt + "," + $symbol | Out-File $quoteIDFileGO -Encoding OEM -Append;
    }
    "" | Out-File $logFile -Encoding OEM -Append;
}

$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed.. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Duration: $duration";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;



