# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Google Web History".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\Quotes\"; if (!(Test-Path -Path $quotesFolder)) {"Quotes folder not found: $quotesFolder"; exit(1)};

$listStart = $config.IndexOf("<GoogleWeb>"); $listEnd = $config.IndexOf("</GoogleWeb>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<GoogleWeb> Symbol list is empty. Exiting script.";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $symbolList.count + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;

$urlBase   = "https://www.google.com/finance/historical?q=@@Symbol@@&num=@@quotesPerPage@@&start=@@StartRow@@&startdate=@@StartDate@@" #https://www.google.com/finance/historical?q=MUTF_CA:TDB976&num=30&start=30&startdate=Jan+1%2C+1990
$mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
ForEach($sLine in $symbolList) { # For each symbol
    $sLineArray = $sLine.Split(","); $symbol = $sLineArray[0]; $symbolMinDate = $sLineArray[1]; $symbolMaxDate = $sLineArray[2]; $lastDateInFile = "";
    $symbolQuoteFile = $quotesFolder + "_" + $symbol.Replace(":","_").Replace("^","_").Replace("&","_") + "_" + ".txt"; # Symbol file were quotes will be saved. Adding "_" in front of symbol names as sometimes sequences like \PRN or \CON means something....
    if ($symbolMinDate -ne "" -and $symbolMinDate -ne $null -and ($symbolMinDate.length -ne 10 -or $symbolMinDate -le "1950-01-01" -or $symbolMinDate -gt "2050-01-01")) {$str=" ***Error. Symbol $symbol MinDate specified is not correct. Value provided '$symbolMinDate'. Expected date in format YYYY-MM-DD"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1); }
    if ($symbolMaxDate -ne "" -and $symbolMaxDate -ne $null -and ($symbolMaxDate.length -ne 10 -or $symbolMaxDate -le "1950-01-01" -or $symbolMaxDate -gt "2050-01-01")) {$str=" ***Error. Symbol $symbol MaxDate specified is not correct. Value provided '$symbolMinDate'. Expected date in format YYYY-MM-DD"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1); }
    if ($symbolMinDate -eq "" -or  $symbolMinDate -eq $null -or  $symbolMinDate -lt $minDate) {$nextDate = $minDate;} else {$nextDate = $symbolMinDate;}
    if ($symbolMaxDate -eq "" -or  $symbolMaxDate -eq $null) {$symbolMaxDate = (Get-Date).AddDays(1).ToString("yyyy-MM-dd");}
    "Symbol: $symbol" | Out-File $logFile -Encoding OEM -Append; $log = (Get-Date).ToString("HH:mm:ss") + " " + $symbol.PadRight(20);

    # Getting date of last quote for symbol in quote file we already have. If there is no quote file, use min date from parameter file
    if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
        $fc = @(import-csv $symbolQuoteFile -Header "Date","Close","Symbol" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;} # Finding max quote date in file.Format will be YYYY-MM-DD
        if ($lastDateInFile -eq $null) {$lastDateInFile = "";}
        if ($lastDateInFile -ne "") { # File exists and max date in the file found
            $nextDate = ([datetime]::ParseExact($lastDateInFile,"yyyy-MM-dd",$null)).AddDays(1).ToString("yyyy-MM-dd"); # Adding one day to max date found in quote file - this will be next request start date
            "  Found quote file. Looking for quotes starting from: $nextDate" | Out-File $logFile -Encoding OEM -Append;
        }
        else {$nextDate = ([datetime]::ParseExact($symbolMaxDate,"yyyy-MM-dd",$null)).AddDays(10).ToString("yyyy-MM-dd");} # Empty file means that extract already got AVAILABLE data up to max date, so no need to load more
    }
    $log += " Last local quote: " + $lastDateInFile.PadRight(10) + ".";
    if ($nextDate -gt $symbolMaxDate) { # For this symbol we already have localy data up to Max date.
        "  We already have data up to maximum configuration date of $symbolMaxDate. Will not request new data for this symbol." | Out-File $logFile -Encoding OEM -Append; 
        #$log + " Local file has all necessary data."; 
        continue; 
    } 
    
    $symbolMaxDateAdj = ([datetime]::ParseExact($symbolMaxDate,"yyyy-MM-dd",$null)).AddDays(10).ToString("yyyy-MM-dd"); # Will try to save up to 10 more days than request, because specific day quote might not be available (ex: weekend)
    $year = $nextDate.Substring(0,4); $month = [int]$nextDate.Substring(5,2) - 1; $day = $nextDate.Substring(8,2); $mthNotShifted = [int] $month+1; # Spliting date into parts. Month is 0 based

    $nextDateGO = $mths[$month]+"+"+[int] $day+"%2C+"+$year; # #$nextDate = '2012-10-01'; $nextDateGO = "Oct+1%2C+2012"
    $symbolGOW = $symbol.Replace("&","%26").Replace(":","%3A");
    
    $tqc=0; # Total Quote count for that symbol
    $qpp=30; #default quotes per page value is 30;
    if ((New-TimeSpan $nextDate).Days -ge 30) {$qpp=200;} #Max quotes per page will be 200
    $urlSymbol = $urlBase.Replace("@@Symbol@@",$symbolGOW).Replace("@@quotesPerPage@@",$qpp).Replace("@@StartDate@@",$nextDateGO)
    $pg=0; # starting with page 0.
    WHILE ($pg -le 50) { #expecting up to a 50 pages. Max days: 50x200=10,000 days = 27Years
        $quoteCount=0; $webpage = ""; $reqCount++;
        $url = $urlSymbol.Replace("@@StartRow@@",($pg*$qpp).ToString());
        "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append;
        $wc = new-object system.net.WebClient;
        try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
        catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; if($verbose) {$log + " No new data received."}; continue;} # if attempt to get webpage failed go to next symbol
        $str = [System.Text.Encoding]::ASCII.GetString($webpage); # This variable now contains downloaded quotes
    
        $startStr = "<table class=`"gf-table historical_price`">"; $endStr = "</table>";
        if (!$str.Contains($startStr)) {
            if ($tqc -eq 0) { $reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no <table class=... in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."};} # if that symbol has no quotes, then record errors
            $pg=100; #Exiting page loop
            continue; # go to next Symbol
        }
        $str = $str.Substring($str.IndexOf($startStr) + $startStr.Length);
        $mfFlag = $true; if ($str.Contains("<th class=`"rgt bb`">Open")) {$mfFlag=$false;}; #page could have "Date,Close" values for Mutual Funds and "Date,Open,High,Low,Close,Volume" for stocks. Finding if this MF or Stock

        $reqSucceed++;
        $d2 = '2100-01-01'; #some future date
        $startStr1 = "<td class=`"lm`">"; $startStr2 = "<td class=`"rgt`">"; $startStr3 = "<td class=`"rgt rm`">"
        while($str.Contains($startStr1) -and $d2 -gt $nextDate) { #quote table row found
            $quoteCount++; 
            $str = $str.Substring($str.IndexOf($startStr1) + $StartStr1.Length); 
            $d = $str.Substring(0, $str.IndexOf("`n")); #Date

            #$d2 = [datetime]::ParseExact($d, "MMM d, yyyy", $null).ToString("yyyy-MM-dd"); # does not work when run on foreign language PC. Google returns Eng month name, PC translates to Dutch/Idian/German/...
            $mthName = $d.Substring(0,3); $qMonth= "0"+($mths.IndexOf($mthName) +1); $qMonth = $qMonth.Substring($qMonth.length-2,2); # converting month name into number: Feb->02
            $qYear = $d.Substring(7).Trim();
            $qDay = "0"+$d.Substring(4,2).Replace(",",""); $qDay = $qDay.Substring($qDay.Length-2, 2);
            $d2 = $qYear + "-" + $qMonth +"-" + $qDay

            if (!$mfFlag) { # just if this not mutual fund and page has Open,High,Low price columns, we need to skip them
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # Open Price - ignoring this column by removing value from our string
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # High Price - ignoring this column by removing value from our string
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # Low Price  - ignoring this column by removing value from our string
            }

            # Close Price - taking data from this column
            $cpStartStr = $startStr2; if ($mfFlag) {$cpStartStr = $startStr3;} # for close price start string is different for stocks and mutual fund
            if (!$str.contains($cpStartStr)) {continue;} #Price not found, continue to next record
            $str = $str.Substring($str.IndexOf($cpStartStr) + $cpStartStr.Length); 
            $cp = $str.Substring(0, $str.IndexOf("`n")).Replace(",","").Trim(); #Close Price
            if ($cp -eq "" -or $cp -eq "-") {continue;} #Value is empty or dash

            #if (!$str.contains($startStr3)) {continue;} $str = $str.Substring($str.IndexOf($startStr3) + $StartStr3.Length); # Volume - ignoring this column by removing value from our string

            if ($d2 -ge $nextDate -and $d2 -le $symbolMaxDateAdj) {
                $d2 + "," + $cp + "," + $symbol | Out-file $symbolQuoteFile -append -Encoding OEM
                if ($d2 -gt $lastQuote) {$lastQuote=$d2;} # Finding latest quote date from new records
                $tqc++; 
            }
        } # while($str.Contains($startStr1)
        if ($quoteCount -lt $qpp) {break;}
        $pg++;
    } #WHILE ($pg -le 1000) {

    if ($tqc -gt 0) {"  Added new quotes to file: " + $symbolQuoteFile | Out-File $logFile -Encoding OEM -Append; }
    "Done: $symbol (from: $nextDate). Record count: $tqc. Last quote: $lastQuote `r`n" | Out-File $logFile -Encoding OEM -Append; 
    if ($tqc -gt 0) {if($verbose) {$log + " New rec count " + $tqc.ToString().PadLeft(4) + ". Last quote: $lastQuote"};}
}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Duration: $duration";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;


