# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
$psDataFolder   = $scriptPathParent + "\PSData"; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Create PS Files".PadRight(28);
$culture = New-Object System.Globalization.CultureInfo("en-US"); # Getting culture for proper formatting of number

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output

$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$psDataFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<PSDataFolder>"))+1)).Replace("</PSDataFolder>","");
if ($psDataFolderCfg -ne $null -and $psDataFolderCfg -ne "") {$psDataFolder = $psDataFolderCfg;} 
if (!(Test-Path $psDataFolder)) {New-Item $psDataFolder -type directory}; "PSData folder: $psDataFolder" | Out-File $logFile -Encoding OEM -Append;

$quotesFolder     = $dataRootFolder + "\Quotes\";    if (!(Test-Path -Path $quotesFolder))   {$str="** Error. Quotes data folder not found: $quotesFolder";   $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1)};
$currDataFolder   = $dataRootFolder + "\CurrExch\";  if (!(Test-Path $currDataFolder)) {New-Item $currDataFolder -type directory};  # Create Data folders
$divDataFolder    = $dataRootFolder + "\Dividends\"; if (!(Test-Path $divDataFolder)) {New-Item $divDataFolder -type directory};  # Create Data folders

$quoteIDFileYA    = $dataRootFolder + "\QuotesIntraDay\YahooIntraday.txt"; # This file might or might not exist
$currExchIDFileYA = $dataRootFolder + "\QuotesIntraDay\YahooCurrExchIntraday.txt";
$quoteIDFileGO    = $dataRootFolder + "\QuotesIntraDay\GoogleIntraday.txt"; # This file might or might not exist

$includeArchiveFlag = $false; $includeArchiveValue = ($config | Select-Object -Index(($config.IndexOf("<IncludeQuoteArchiveFolder>"))+1)).Replace("</IncludeQuoteArchiveFolder>",""); # Getting IncludeArchive value
if ($includeArchiveValue -ne $null) {if ($includeArchiveValue.Trim().ToLower() -eq "yes") {$includeArchiveFlag = $true;}} 

$colSepTxt = ($config | Select-Object -Index(($config.IndexOf("<ColumnSeparator>"))+1)).Replace("</ColumnSeparator>",""); $colSep = "`t";
if ($colSepTxt.ToLower() -eq "comma") {$colSep = ","}; if ($colSepTxt.ToLower() -eq "tab") {$colSep = "`t"}; if ($colSepTxt.ToLower() -eq "verticalbar") {$colSep = "|"};

$decSep = "."; $decSep = ($config | Select-Object -Index(($config.IndexOf("<DecimalSeparator>"))+1)).Replace("</ColumnSeparator>",""); $decSep = $decSep.Substring(0, 1);

$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name; if ($verbose) {$str};

# #######################################################################################
# First loading all factors (History, Intraday and Dividends into array for fast access
# #######################################################################################
# ============ Building array of factors for History,Intraday (quotes) and dividends. $sFH - array of symbols with historical factors, $sFHF - array of factors for each symbol in $sFH, etc.
$sFH = @(); $sFHF = @(); $sFI = @(); $sFIF = @(); $sDI = @(); $sDIF = @();
$listStart = $config.IndexOf("<Yahoo>"); $listEnd = $config.IndexOf("</Yahoo>");  #Symbol,MinDate,MaxDate,IntraDayFlag[Y|N],[DividendFlag[Y|N],FactorHistory,FactorIntraDay,FactorDividend
if (!($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd)) {
    $list = @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));
    ForEach($line in $list) { $a = $line.Split(",");
        if($a[5] -ne "" -and $a[5] -ne $null) {$sFH += ,$a[0]; $sFHF += ,$a[5];} # Building arrays for FactorHistory
        if($a[6] -ne "" -and $a[6] -ne $null) {$sFI += ,$a[0]; $sFIF += ,$a[6];} # Building arrays for FactorIntraday
        if($a[7] -ne "" -and $a[7] -ne $null) {$sDI += ,$a[0]; $sDIF += ,$a[7];} # Building arrays for DividendFactor
}   }

$listStart = $config.IndexOf("<Google>"); $listEnd = $config.IndexOf("</Google>");  #Symbol,MinDate,MaxDate,IntraDayFlag[Y|N],[DividendFlag[Y|N],FactorHistory,FactorIntraDay,FactorDividend
if (!($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd)) {
    $list = @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));
    ForEach($line in $list) { $a = $line.Split(",");
        if($a[5] -ne "" -and $a[5] -ne $null) {$sFH += ,$a[0]; $sFHF += ,$a[5];} # Building arrays for FactorHistory
        if($a[6] -ne "" -and $a[6] -ne $null) {$sFI += ,$a[0]; $sFIF += ,$a[6];} # Building arrays for FactorIntraday
        if($a[7] -ne "" -and $a[7] -ne $null) {$sDI += ,$a[0]; $sDIF += ,$a[7];} # Building arrays for DividendFactor
}   }

$listStart = $config.IndexOf("<GoogleWeb>"); $listEnd = $config.IndexOf("</GoogleWeb>");  #Symbol,MinDate,MaxDate,IntraDayFlag[Y|N],[DividendFlag[Y|N],FactorHistory,FactorIntraDay,FactorDividend
if (!($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd)) {
    $list = @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));
    ForEach($line in $list) { $a = $line.Split(",");
        if($a[5] -ne "" -and $a[5] -ne $null) {$sFH += ,$a[0]; $sFHF += ,$a[5];} # Building arrays for FactorHistory
        if($a[6] -ne "" -and $a[6] -ne $null) {$sFI += ,$a[0]; $sFIF += ,$a[6];} # Building arrays for FactorIntraday
        if($a[7] -ne "" -and $a[7] -ne $null) {$sDI += ,$a[0]; $sDIF += ,$a[7];} # Building arrays for DividendFactor
}   }

$listStart = $config.IndexOf("<Ariva>"); $listEnd = $config.IndexOf("</Ariva>");  # Format: Symbol, SymbolCode, MinDate, MaxDate,FactorHistory
if (!($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd)) {
    $list = @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));
    ForEach($line in $list) {
        $a = $line.Split(",");
        if($a[4] -ne "" -and $a[4] -ne $null) {$sFH += ,$a[0]; $sFHF += ,$a[4];} # Building arrays for FactorHistory
}   }

# #######################################################################################
# #######################################################################################
# #######################################################################################


# #######################################################################################
# ######################## Creating Currency Exchange file.
# #######################################################################################
$outFile = $psDataFolder + "\CurrencyConv.csv"; $fileCount = 0; $recCount = 0;
"Date,ExchRate,CurrencyFrom,CurrencyTo".Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM;
("$minDate,1.0000000001,N/A,N/A").Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM -Append; 
(Get-ChildItem $currDataFolder | where {$_.extension -eq ".txt"}) | ?{$fc=@(Get-Content -Path $_.FullName); $fileCount+=1; $recCount+=$fc.Count; $fc.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append };
$currDataFolder = $currDataFolder + "\Manual";
if (Test-Path -Path $currDataFolder) { # if Manual subfolder exists, then include all txt files form that subfolder
    (Get-ChildItem $currDataFolder | where {$_.extension -eq ".txt"}) | ?{$fc=@(Get-Content -Path $_.FullName | ? {$_.Trim() -ne ""}); $fileCount+=1; $recCount+=$fc.Count; $fc.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append };
}
$str=(Get-Date).ToString("HH:mm:ss") + " Finished creating file CurrencyConv.csv. Source file count: " + $fileCount.ToString().PadLeft(3) + ". Record count: $recCount.";
$str | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$str};

if (Test-Path $currExchIDFileYA) {(Get-Content -Path $currExchIDFileYA).Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;} # Adding intraday quotes if file found
# #######################################################################################




# #######################################################################################
# ========================== Creating dividends file
# #######################################################################################
$outFile = $psDataFolder + "\Dividends.csv"; $fileCount = 0; $recCount = 0;
"PayDate,DividendPerShare,Symbol".Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM; # File should always have at least a header
("$minDate,0.0000000001,* Cash").Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM -Append; # Adding second line with todays date to enforce data types in Excel.Tiny amount will not be visible
$fl = (Get-ChildItem $divDataFolder | where {$_.extension -eq ".txt"}); if ($fl.count -eq 1) {$fl = ,@($fl)}; # Reading all *.txt files from dividends data folder. If 1 record returned, convert to array so we can add other folders
if (Test-Path ($divDataFolder+"\Manual")) {
    $fl += (Get-ChildItem ($divDataFolder+"\Manual") | where {$_.extension -eq ".txt"}); # Including "\Manual" subfolder
}
ForEach($f in $fl) { # working on each file ($f)
    $fc=@(Get-Content -Path $f.FullName | ? {$_.Trim() -ne ""}); if ($fc.Count -le 0) {continue;} $fileCount+=1; $recCount+=$fc.Count; $symbol = $fc.Split(",")[2];   # Get file content into $fc
    if ($sDI.IndexOf($symbol) -ne -1) { $factor = [double]$sDIF[$sDI.IndexOf($symbol)]; # if symbol located in array, then on dividend amount we need to apply factor
        ForEach($dl in $fc) {$d = $dl.Split(","); ($d[0] + "," + (([double]$d[1]) * $factor).ToString("########0.0######", $culture) + "," + $d[2]).Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;} #output each row with factor applied
    }
    else {$fc.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;} # just dump file content into final file without adjusting amounts
}
$str=(Get-Date).ToString("HH:mm:ss") + " Finished creating file    Dividends.csv. Source file count: " + $fileCount.ToString().PadLeft(3) + ". Record count: $recCount.";
$str | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$str};
# #######################################################################################

# #######################################################################################
# ######################## Creating Quotes file
# #######################################################################################
$outFile = $psDataFolder + "\Quotes.csv"; $fileCount = 0; $recCount = 0;
"Date,Close,Symbol".Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM;
("$minDate,1.0000000001,* Cash").Replace(",", $colSep).Replace(".",$decSep) | Out-File $outFile -Encoding OEM -Append; 
$fl = @(Get-ChildItem $quotesFolder -Recurse | where {$_.extension -eq ".txt" -and (!($_.BaseName -like "*_Archive") -or $includeArchiveFlag)}); #if ($fl.count -eq 1) {$fl = ,@($fl)};
ForEach($f in $fl) {
    $fc=@(Get-Content -Path $f.FullName | ? {$_.Trim() -ne ""}); if ($fc.Count -le 0) {continue;} $fileCount+=1; $recCount+=$fc.Count; $symbol = $fc.Split(",")[2];   # Get file content into $fc
    if ($sFH.IndexOf($symbol) -ne -1) { $factor = [double]$sFHF[$sFH.IndexOf($symbol)]; # if symbol located in array, then on amount we need to apply factor
        ForEach($dl in $fc) {$d = $dl.Split(","); ($d[0] + "," + (([double]$d[1]) * $factor).ToString("#########0.0#####", $culture) + "," + $d[2]).Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;} #output each row with factor applied
    }
    else {$fc.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;} # just dump file content into final file without adjusting amounts
}
$str=(Get-Date).ToString("HH:mm:ss") + " Finished creating file       Quotes.csv. Source file count: " + $fileCount.ToString().PadLeft(3) + ". Record count: $recCount.";
$str | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$str};
# #######################################################################################

# #######################################################################################
# ######################## Adding intraday quotes
# #######################################################################################
$outFile = $psDataFolder + "\Quotes.csv"; $fileCount = 0; $recCount = 0;
if (Test-Path $quoteIDFileYA) { # if Yahoo intraday file exists
    $fc = @(Get-Content -Path $quoteIDFileYA);
    ForEach($l in $fc) {
        $ls = $l.split(","); $symbol = $ls[2];
        if ($sFI.IndexOf($symbol) -ne -1) { $factor = [double]$sFIF[$sFI.IndexOf($symbol)]; # if symbol located in array, then on amount we need to apply factor
            ($ls[0] + "," +  (([double]$ls[1]) * $factor).ToString("##########0.0#####", $culture) + "," + $ls[2]).Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;
        }
        else {$l.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;}
    }
}
if (Test-Path $quoteIDFileGO) { # if Google intraday file exists
    $fc = @(Get-Content -Path $quoteIDFileGO);
    ForEach($l in $fc) {
        $ls = $l.split(","); $symbol = $ls[2];
        if ($sFI.IndexOf($symbol) -ne -1) { $factor = [double]$sFIF[$sFI.IndexOf($symbol)]; # if symbol located in array, then on amount we need to apply factor
            ($ls[0] + "," +  (([double]$ls[1]) * $factor).ToString("##########0.0#####", $culture) + "," + $ls[2]).Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;
        }
        else {$l.Replace(",", $colSep).Replace(".",$decSep) | Out-file $outFile -Encoding OEM -Append;}
    }
}
# #######################################################################################

# #######################################################################################
# ######################## Creating file schema.ini if column separator is TAB
# #######################################################################################
if ($colSepTxt.ToLower() -eq "tab") {
    $schemaFile = "[CurrencyConv.csv]`r`nFormat=TabDelimited`r`n`r`n[Dates.csv]`r`nFormat=TabDelimited`r`n`r`n[Dividends.csv]`r`nFormat=TabDelimited`r`n`r`n[Quotes.csv]`r`nFormat=TabDelimited`r`n";
    $schemaFile | Out-file ($psDataFolder+"\schema.ini") -Encoding OEM;
}


# #######################################################################################
# ######################## Symbol rename for Quotes.csv and Dividends.csv files
# #######################################################################################
$qFileName = $psDataFolder + "\Quotes.csv"; $dFileName = $psDataFolder + "\Dividends.csv"; $renameCount = 0;
if ((Test-Path $qFileName) -and (Test-Path $dFileName)) {
    $listStart = $config.IndexOf("<SymbolRename>"); $listEnd = $config.IndexOf("</SymbolRename>");  
    if (!($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd)) {
        $qFile = @(Get-Content -Path $qFileName); $dFile = @(Get-Content -Path $dFileName);
        $list = @($config | Select-Object -Index(($listStart+1)..($listEnd-1)));
        ForEach($line in $list) { 
            $a = $line.Split(",");
            if($a[0] -ne "" -and $a[0] -ne $null -and $a[1] -ne "" -and $a[1] -ne $null) {
                $oldSymb = $colSep + $a[0]+"$"; $newSymb = $colSep + $a[1]; $renameCount++; # adding $ to end to specify that change can be done just at the end of the line.
                $qFile = $qFile -replace $oldSymb, $newSymb;
                $dFile = $dFile -replace $oldSymb, $newSymb;
            } #if
        } #ForEach
        $qFile | Out-file $qFileName -Encoding OEM -Force;  
        $dFile | Out-file $dFileName -Encoding OEM -Force;  
    }
}
